;=====================================================================
; LOOP3_16.ASM
; Keith Larson
; TMS320 DSP Applications
; (C) Copyright 2001
; Texas Instruments Incorporated
;
; This is a simple loopback test for the PCM3003 daughter card using
; the TMS320C31 DSK.  The card should be configured for 16 bit
; transfers using the jumper listings given below.
;
; Since transfers occur at the LRCIN rate it is relatively simple
; to configure the serial port for fixed burst mode operation.
; The only factor to consider is that since each signal from the
; daughter card is inverted by the '04 buffers.
;
; Additional notes
; 1) L/R input is summed with an internaly generated sine wave
; 2) You may wish to scale DAC output sum by 50%
; 3) The '.void' directives are new to version 1.27 DSK3 Assembler
; 4) You will want to put a piece of paper between the DC and DSK
;    as the RCA jacks can short to the bottom of the PCM3003 DC
; 5) It appears that if you run the C31 above 64 mhz (I used 80)
;    and you use the on board oscillator, the transfers get hosed.
;    This appears to be related to the slew rate of the signals coming
;    into the DSP (less because of 100 ohm RS values). Transfers work
;    as expected if the DSPs internal TCLK0 is used as the MCLK reference.
; 6) Be sure to use version 1.27 of the DSK3D debugger.  A tweek
;    was made for more reliable restarts during debug
; 7) A mechanism has been added to determine if the serial port
;    has become unsynchronized and needs re-intitialization.
;
;     TCLK1 is now used as a watchdog.  If the serial port
;     hangs and no interrupts occur, TCLK1 will fire
;
; MCLK=24.5 192 khz 96 khz 48 khz
; MCLK=20.0 156 khz 78 khz 39 khz
; MCLK=12.5  98 khz 49 khz 24 khz  (C31-50 DSP TCLK0)
; MCLK=10.0  78 khz 39 khz 20 khz
;
; JP4  1- 2  short  short  short   Enable 20 bit data (32 bit transfers)
;      3- 4                        DEM0
;      5- 6  short  short  short   DEM1   (De-emphisis disabled)
;
; JP5  1- 2                        MCLK = DSP TCLK0
;      3- 4  short  short  short   MCLK = On board oscillator
;      5- 6  short                 SCLK = MCLK/2
;      7- 8         short          SCLK = MCLK/4
;      9-10                short   SCLK = MCLK/8
;     11-12                        SCLK = MCLK/16
;     13-14                        SCLK = MCLK/32
;
; JP6  1- 2                        LRCIN = MCLK/64  (156 khz)
;      3- 4  short                 LRCIN = MCLK/128 ( 78 khz)
;      5- 6         short          LRCIN = MCLK/256 ( 39 khz)
;      7- 8                short   LRCIN = MCLK/256 ( 20 khz)
;      9-10                        LRCIN = MCLK/256 ( 10 khz)
;     11-12                        LRCIN = MCLK/256 (  5 khz)
;     13-14                        LRCIN = MCLK/256 (  2 khz)
;
; ** Other configurations may work with varying degrees of success.
;    Howevre AIC limitations will not work at all with some
;    configurations
;
; This is unsupported freeware with no implied warranties
; or liabilities.  See the disclaimer document for details
;
;=====================================================================
             .include "C3XMMRS.ASM"       ;
S0_xc        .set     0x00000111          ; (last two bits are noise)
S0_rc        .set     0x00000111          ;
XINT_F       .set     0x10                ;
PI           .set     3.14159265          ;
;----------------------------------------------------------------------
;
;          VAREN=0 (fixed)        VAREN=1 (variable)
;          ________                _________ _________
; --------X__DATA__X------     ---X__LDATA__X__RDATA__X---- VAREN=1
; _______--xxxxxxxxx______     ____--xxxxxxxx__________--xx
;        ^
; data transfers 1 sclk after  data transfers when frame becomes
; frame is received            active (does not need to remain active)
;
;----------------------------------------------------------------------
XCLKSRCE     .set     0  << 6             ; 0=external 1= internal
RCLKSRCE     .set     0  << 7             ;
XVAREN       .set     1  << 8             ; VAREN  0       1
RVAREN       .set     1  << 9             ; FS ____---------_____
XFSM         .set     0  <<10             ; 0=burst 1=continuous
RFSM         .set     0  <<11             ;
CLKXP        .set     0  <<12             ;
CLKRP        .set     0  <<13             ;
DXP          .set     0  <<14             ;
DRP          .set     0  <<15             ;
FSXP         .set     1  <<16             ;
FSRP         .set     1  <<17             ;
XLEN         .set     11b<<18             ;
RLEN         .set     11b<<20             ;
EXTINT       .set     0  <<22             ;
EXINT        .set     1  <<23             ;
ERTINT       .set     0  <<24             ;
ERINT        .set     1  <<25             ;

S0         .sdef  XCLKSRCE|RCLKSRCE|XVAREN|RVAREN|XFSM|RFSM|CLKXP|CLKRP
S0         .sdef  S0|DXP|DRP|FSXP|FSRP|XLEN|RLEN|EXTINT|EXINT|ERTINT|ERINT
S0_grst    .word    S0
S0_grun    .word    S0|0x0C000000
S0_reset     .word    0x0C000000
;-----------------------------------------
Freq      .set    0.0100100              ; F as fraction of sample rate
SGPTR     .word   SR                     ; pointers to siggen structures
STR       .float  cos(Freq *2*PI)        ;
SR        .float  1.0                    ; REAL vector
SI        .float  0.0                    ; IMAG vector
STI       .float  sin(Freq *2*PI)        ;
SCALE     .float      0x8000             ; +/- 2^15
INVSCALE  .float  1.0/0x8000             ;
CLIP      .float  0x00007FFE             ;
Frin      .float  0                      ;
Flin      .float  0                      ;
;=======================================================================
          .entry  MAIN                   ;
MAIN      ldp     stack                  ; Load DP
          ldi     @stack,SP              ; init stack pointer
          or      0x200,IF
;----------------------------------------
NEXT      ldi     0x234,IE               ; Enable TINT1
          idle                           ;
          ldi     0,R0                   ; Reset the watchdog timer
          sti     R0,@T1_count           ; If the debugger corrupts the
          b       NEXT                   ; serial port, XINT will stop
;----------------------------------------  and the timer will fire instead
XINT      push    ST                     ;
          push    R0                     ;
          pushf   R0                     ;
          push    R2                     ;
          pushf   R2                     ;
          call    SIGGEN                 ;
          ldf     @SR,R0                 ;
          addf    @Flin,R0               ;
          call    FLOAT2DAC              ;
          ldi     R0,R2                  ;
          ldf     @SI,R0                 ;
          addf    @Frin,R0               ;
          call    FLOAT2DAC              ;
          lsh     -16,R0                 ;
          or      R2,R0                  ;
          sti     R0,@S0_xdata           ; transmit L+R to DAC
          popf    R2                     ;
          pop     R2                     ;
          popf    R0                     ;
          pop     R0                     ;
          pop     ST                     ;
          rets                           ;
;----------------------------------------
RINT      push    ST                     ;
          push    R0                     ;
          pushf   R0                     ;
          ldi     @S0_rdata,R0           ; Get ADC input
          ash     -16,R0                 ;
          float   R0,R0                  ;
          mpyf    @INVSCALE,R0           ;
          stf     R0,@Frin               ;
          ldi     @S0_rdata,R0           ; Re-get ADC input from SP
          lsh     16,R0                  ;
          ash     -16,R0                 ;
          float   R0,R0                  ;
          mpyf    @INVSCALE,R0           ;
          stf     R0,@Flin               ;
          popf    R0                     ;
          pop     R0                     ;
          pop     ST                     ;
          rets                           ;
;----------------------------------------
FLOAT2DAC mpyf    0.5,R0                 ;
          mpyf    @SCALE,R0              ; Scale +/-1 float to integer DAC
          cmpf    @CLIP,R0               ;
          ldfge   @CLIP,R0               ;
          negf    R0,R0                  ;
          cmpf    @CLIP,R0               ;
          ldfge   @CLIP,R0               ;
          negf    R0,R0                  ;
          fix     R0,R0                  ; return in upper 16 bits
          lsh     16,R0                  ;
          rets                           ;
;-----------------------------------------
TINT1     push    ST
          push    R0
          ;---------------------
INIT_TX   ldi     0,R0
          sti     R0,@T0_ctrl
          sti     R0,@T1_ctrl
          sti     R0,@T0_count
          sti     R0,@T1_count
          sti     R0,@T0_prd
          ldi     25E6/1E3,R0
          sti     R0,@T1_prd
          ldi     0x3C1,R0
          sti     R0,@T0_ctrl
          sti     R0,@T1_ctrl
          ;---------------------
INIT_SP   ldi     S0_xc,R0               ; Init serial port
          sti     R0,@S0_xctrl           ;
          ldi     S0_rc,R0               ;
          sti     R0,@S0_rctrl           ;
FSHI      ldi     @S0_rctrl,R0           ; Wait for LRCIN=1 (  active)
          and     0x800,R0               ;
          bz      FSHI                   ;
          ldi     @S0_grst,R0            ; Reset serial port
          sti     R0,@S0_gctrl           ;
FSLO      ldi     @S0_rctrl,R0           ; Wait for LRCIN=0 (inactive)
          and     0x800,R0               ;
          bnz     FSLO                   ;
          ldi     @S0_grun,R0            ;
          sti     R0,@S0_gctrl           ;
          ldi     0,IF                   ;
          sti     R0,@S0_xdata
          pop     R0
          pop     ST
          rets
;=========================================================================
; SIGGEN uses a complex multiply to phase rotate a signal vector.  The
; result is a sampled +/-1 magnitude sine wave which is returned in R0
;=========================================================================
SIGGEN    push    AR7                    ;
          ldi     @SGPTR ,AR7            ; Compute next sine/cosine points
          mpyf    *AR7++(1), *-AR7(1),R0 ; STR*SR
          mpyf    *AR7     , *+AR7(1),R2 ; STI*SI
          mpyf    *+AR7(1) ,*--AR7(1),R0 ; STI*SR
     ||   subf    R2,R0,R2               ; RL = STR*SR-STI*SI
          stf     R2,*AR7                ;
          mpyf    *+AR7(1) , *-AR7(1),R2 ; STR*SI
          addf    R0,R2,R2               ; IM = STR*SI+STI*SR
          stf     R2,*+AR7(1)            ;
          ;- - - - - - - - - - - - - - -
ERR_FBK   mpyf    *+AR7(1),*+AR7(1),R2   ; Compute MAG^2
          mpyf    *+AR7(0),*+AR7(0),R0   ;
          addf    R0,R2                  ; R2=MAG^2
          subrf   3,R2                   ;
          mpyf    0.5,R2                 ; Feedback R2=1/sqrt(x)~=(3-x)/2
          mpyf    R2,*AR7,R0             ; Correct amplitude and save
          mpyf    R2,*+AR7(1),R0         ;
      ||  stf     R0,*AR7                ;
          stf     R0,*+AR7(1)            ;
          pop     AR7                    ;
          rets                           ;
;-----------------------------------------
stack     .word   stack
;======================================================================
; Since the C31 is being used in bootloader mode, interrupts begin
; execution at the secondary branch table in internal SRAM.  A branch
; to XINT/RINT ISR routine is placed directly into these locations
;======================================================================
          .start   "VECTORS",0x809FC1
          .sect    "VECTORS"
          .void 1               ; INT0
          .void 1               ; INT1
          .void 1               ; INT2
          .void 1               ; INT3
          br    XINT            ; XINT0
          br    RINT            ; RINT0
          .void 1               ; XINT1
          .void 1               ; RINT1
          .void 1               ; TINT0
          br    TINT1           ; TINT1
          .end


